 /*
 +---------------------------------------------------------------------------+
 | Facebook Microsoft Vista Sidebar Gadget                                   |
 +---------------------------------------------------------------------------+
 | Copyright (c) 2007 Facebook, Inc.                                         |
 | All rights reserved.                                                      |
 |                                                                           |
 | Redistribution and use in source and binary forms, with or without        |
 | modification, are permitted provided that the following conditions        |
 | are met:                                                                  |
 |                                                                           |
 | 1. Redistributions of source code must retain the above copyright         |
 |    notice, this list of conditions and the following disclaimer.          |
 | 2. Redistributions in binary form must reproduce the above copyright      |
 |    notice, this list of conditions and the following disclaimer in the    |
 |    documentation and/or other materials provided with the distribution.   |
 |                                                                           |
 | THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR      |
 | IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES |
 | OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.   |
 | IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,          |
 | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  |
 | NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, |
 | DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     |
 | THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       |
 | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF  |
 | THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.         |
 +---------------------------------------------------------------------------+
 | Developer: Daniel Ruspini - facebook id dan@ruspini.com                   |
 +---------------------------------------------------------------------------+
*/

/*
 *  Manager is the function literal controlling facebook gadget
 *  operations across multiple sub-apps.
 */
Manager = new function() {

	this.stopped = false;
	this.dockedDimensions = { width:138, height:138 };
	this.undockedDimensions = { width:348, height:262 };
	this.internetConnectionTimer;
	this.internetConnectionTimerDelay = 10000;


	/*
	 *  To be called when gadget is loaded and page ready
	 */
	this.loadMain = function() {
        Util.printGadgetInfo();
		this.initializeGadget();
		this.checkDockedState();
		this.copyBookmarklet();

		FacebookAPI.restoreState();
		if (FacebookAPI.isLoggedIn()) {
		    this.startGadget();
		}
	}
	
	/*
	 *  Copy the bookmarklet url to the user's favorites folder
	 */
	this.copyBookmarklet = function() {
	    try {
            var fso = new ActiveXObject("Scripting.FileSystemObject");
            var source = System.Gadget.path + "\\en-US\\url\\Share on Facebook.url";
            var destination = System.Environment.getEnvironmentVariable("HOMEPATH") + "\\Favorites\\Links\\";

            fso.CopyFile(source, destination, true);
            
            Util.debug("facebook Bookmarklet copied from \""+ source + "\" to \"" + destination + "\"");
        } catch (ex) {
            Util.debug("copyBookmarklet error="+ex);
        }
	}
	
	
	/*
	 *  Hook up events and initialize components of gadget
	 */
	this.initializeGadget = function() {
		System.Gadget.onUndock = chkDockState;
		System.Gadget.onDock = chkDockState;
		System.Gadget.onSettingsClosed = updateGadget;
		System.Gadget.settingsUI = "settings.html";
		Settings.initialize();
		SlideShow.initialize(Settings);
		Poke.initialize();

		Settings.loadSettings();
		this.updateSlideshow();
		this.updatePoke();
	}
	
	/*
	 *  Starts all gadget apps, will be called after authentication
	 */
	this.startGadget = function() {
	    try
	    {
	        if (AjaxUtil.ping() == false)
	            throw "Cannot connect to facebook";

			FacebookAPI.restoreState();
			if (!FacebookAPI.isLoggedIn()) {   
	        	FacebookAPI.createSession(document.getElementById("authToken").value);
	        }
	        this.stopped = false;
		    this.startSlideshow();
		    Poke.start();
		    Upload.start();
		    System.Gadget.Settings.write("gadgetLoggedOut", false);
	    }
	    catch (ex)
	    {
			var error = ex;

			if (ex == "Invalid parameter (100)")
				error = "Invalid code";
		    
			document.getElementById("login-error").innerHTML = error;
			document.getElementById("login-error").style.display = "";
			System.Gadget.Settings.write("gadgetLoggedOut", true);
			document.getElementById("login-btn").disabled = false;
	    }
	}
	
	/*
	 *  Stops all gadget apps.
	 *  Called when user logs out or loss of internet connection.
	 */
	this.stopGadget = function() {
		this.stopped = true;
		SlideShow.stop();
		SlideShow.hide();
		Poke.stop();
		Upload.stop();
		Upload.hide();
	}
	
	/*
	 *  Called when user closes Settings.
	 *  Updates the gadget to reflect new settings, including logging out.
	 */
	this.updateGadget = function() {
		Settings.loadSettings();
		this.updateSlideshow();
		this.updatePoke();
		
		if (System.Gadget.Settings.read("gadgetLoggedOut")) {
			FacebookAPI.clearState();
			Manager.stopGadget();
			Login.show();
		}
	}
	
	/*
	 *  Starts slideshow application
	 */
	this.startSlideshow = function() {
		SlideShow.loadPhotos();
		SlideShow.show();;
		Login.hide();
		Upload.hide();
	}
	
	/*
	 *  Continues running the slideshow app after having been stopped.  
	 *  This happens when you "Continue Slideshow" after image upload.
	 */
	this.continueSlideshow = function() {
		Upload.hide();
		Flyout.hide();
		SlideShow.show();
		SlideShow.start();
	}
	
	/*
	 *  Update slideshow app to reflect new settings
	 */ 
	this.updateSlideshow = function() {
		if (SlideShow.delay != Settings.slideshowDelay) {
			Util.debug("[Manager:updateSlideshow]  Changing delay to: " + Settings.slideshowDelay);
			SlideShow.delay = Settings.slideshowDelay;
		}
		if (SlideShow.transition != Settings.slideshowTransition) {
			Util.debug("[Manager:updateSlideshow]  Changing transition to: " + Settings.slideshowTransition);
			SlideShow.transition = Settings.slideshowTransition;
		}
		if (SlideShow.imageStoreType != Settings.slideshowImageStore) {
			Util.debug("[Manager:updateSlideshow]  Changing image stores to: " + Settings.slideshowImageStore);
			SlideShow.imageStoreType = Settings.slideshowImageStore;
			SlideShow.loadPhotos();
		}
		if (SlideShow.isCropped != Settings.slideshowCropPhotos) {
			SlideShow.isCropped = Settings.slideshowCropPhotos;
			SlideShow.resizeCurrentImage();
		}
	}
	
	this.startUpload = function() {
		Login.hide();
		SlideShow.stop();
		SlideShow.hide();
		Upload.show();
	}
	
	
	/*
	 *  Update poke app to reflect new settings
	 */ 
	this.updatePoke = function() {
		if (Poke.showPokeAlert != Settings.showPokeAlert) {
			Util.debug("[Manager:updatePoke]  Changing showPokeAlert to: " + Settings.showPokeAlert);
			Poke.showPokeAlert = Settings.showPokeAlert;
		}
	}
	
	this.hasInternetConnection = function() {
		try {
			if(AjaxUtil.ping()) {
				Util.debug("Internet connection up");
				
				FacebookAPI.restoreState();
				if (FacebookAPI.isLoggedIn() && this.stopped) {
				//if (this.stopped) {
					this.startGadget();
				}
				this.hideCheckInternetConnectionPane();
				return true;
			}
		} catch (ex) {
			Util.debug("Internet connection down");
			this.stopGadget();
			this.showCheckInternetConnectionPane();
			return false;
		}
	}
	
	this.startPeriodicalConnectionCheck = function() {
		Util.debug("do periodicalConnectionCheck");
		clearTimeout(this.internetConnectionTimer);
		if (!this.hasInternetConnection()) {
			this.internetConnectionTimer = setTimeout("Manager.startPeriodicalConnectionCheck()", this.internetConnectionTimerDelay);
		} else {
			clearTimeout(this.internetConnectionTimer);
		}
	}
	
	this.hideCheckInternetConnectionPane = function() {
		document.getElementById("check-connection").style.display = "none";
	}
	
	this.showCheckInternetConnectionPane = function() {
		document.getElementById("check-connection").style.display = "";
	}
	
	this.startLogin = function() {
		Login.show();
		SlideShow.hide();
		Upload.hide();
		this.hideCheckInternetConnectionPane();
	}
	
	/*
	 *  Event when poke indicator clicked
	 */
	 this.onClickPoke = function() {
	 	window.open("http://www.facebook.com/home.php");
	 }
	 
	 /*
	  *  Event when Share button is clicked
	  */
	 this.onClickShare = function() {
	 	SlideShow.share();
	 }
	 
	
	/*
	 *  Event for dock and undock
	 */
	this.checkDockedState = function() {
		if (!System.Gadget.docked) {
			Util.debug("gadget undocked!");
			document.body.className = "undocked";
			document.body.style.width = this.undockedDimensions.width;
			document.body.style.height = this.undockedDimensions.height;
			SlideShow.undock();
			Upload.undock();
		} else {
			Util.debug("gadget docked!");
			document.body.className = "docked";
			document.body.style.width = this.dockedDimensions.width;
			document.body.style.height = this.dockedDimensions.height;
			SlideShow.dock();
			Upload.dock();
		}
		document.getElementById("welcome-text").innerHTML = Util.getMessage("WecomeText");
	}
	
	/*
	 *  Drag and drop events
	 */
	this.onDragDropped = function() {
		if (!Upload.stopped) {
			$("#imgWrapper").removeClass("drag-over");
			$("#slideshow").removeClass("drag-over");
			$("#upload-results").removeClass("drag-over");
			$("#upload").removeClass("drag-over");
		}
		DragAndDrop.onDrop();
	}
	
	this.onDragOver = function() {
		if (!Upload.stopped) {
			$("#imgWrapper").addClass("drag-over");
			$("#slideshow").addClass("drag-over");
			$("#upload-results").addClass("drag-over");
			$("#upload").addClass("drag-over");
			event.returnValue = false;
		}
	}
	
	this.onDragOut = function() {
		if (!Upload.stopped) {
			$("#imgWrapper").removeClass("drag-over");
			$("#slideshow").removeClass("drag-over");
			$("#upload-results").removeClass("drag-over");
			$("#upload").removeClass("drag-over");
		}
	}
	
	this.onMouseOver = function() {
		SlideShow.onMouseOver();
	}
	
	this.onMouseOut = function() {
		SlideShow.onMouseOut();
	}
	
	this.onDblClick = function() {
		SlideShow.openInWindow();
	}
	
	this.onClickLogo = function() {
		window.open("http://www.facebook.com/home.php");
	}
	
}

function chkDockState() {
	Manager.checkDockedState();
}

function updateGadget() {
	if (System.Gadget.Settings.read("settingsCommitted")) {
		Manager.updateGadget();
	}
}