 /*
 +---------------------------------------------------------------------------+
 | Facebook Microsoft Vista Sidebar Gadget                                   |
 +---------------------------------------------------------------------------+
 | Copyright (c) 2007 Facebook, Inc.                                         |
 | All rights reserved.                                                      |
 |                                                                           |
 | Redistribution and use in source and binary forms, with or without        |
 | modification, are permitted provided that the following conditions        |
 | are met:                                                                  |
 |                                                                           |
 | 1. Redistributions of source code must retain the above copyright         |
 |    notice, this list of conditions and the following disclaimer.          |
 | 2. Redistributions in binary form must reproduce the above copyright      |
 |    notice, this list of conditions and the following disclaimer in the    |
 |    documentation and/or other materials provided with the distribution.   |
 |                                                                           |
 | THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR      |
 | IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES |
 | OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.   |
 | IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,          |
 | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  |
 | NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, |
 | DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     |
 | THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       |
 | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF  |
 | THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.         |
 +---------------------------------------------------------------------------+
 | Developer: Daniel Ruspini - facebook id dan@ruspini.com                   |
 +---------------------------------------------------------------------------+
*/

/*
 * Poke is the function literal controlling facebook poke UI.
 */
Poke = new function() {

	this.notification;
	this.delay = 60000;		// check for new poke interval time
	this.oldMostRecent;
	this.timer;
	this.stopped = false;
	this.showPokeAlert = true;
	
	// shake variables
	this.delta = 4;
	this.speed = 25;
	this.effect = "linear";
	this.startPos = { x:4, y:4 };	// should match #gadget-app (left & top) in css
	this.numCycles = 4;
	this.cycleCount = 1;
	

	this.initialize = function() {
		this.oldMostRecent = System.Gadget.Settings.read("oldMostRecent");
	}
	
	this.load = function() {
		if (Manager.hasInternetConnection()) {
			this.notification = FacebookAPI.getNotification();
			this.updateUI();
		} else {
        	Manager.startPeriodicalConnectionCheck();
        }
	}
	
	this.start = function() {
		this.stopped = false;
		this.load();
		this.startTimer();
	}
	
	this.stop = function() {
		this.stopped = true;
		clearInterval(this.timer);
	}
	
	this.startTimer = function() {
		clearInterval(this.timer);
		if (this.stopped) { return; }
		this.timer = setInterval("Poke.load()", this.delay);
	}	
	
	this.updateUI = function() {
		document.getElementById("poke-unread-count").innerHTML = "(" + this.notification.unreadPokes + ")";
		if (this.notification.unreadPokes > 0 && this.notification.mostRecentPokes > this.oldMostRecent) {
			this.oldMostRecent = this.notification.mostRecentPokes;
			System.Gadget.Settings.write("oldMostRecent", this.oldMostRecent);
			this.doShake();
		}
		this.oldMostRecent = this.notification.mostRecentPokes;
	}
	
	/*
	 *  Produces the shake effect to the gadget based on an animation
	 *  that runs along a star pattern.  
	 *  Based on parameters: delta, speed, effect, numCycles.
	 */
	this.doShake = function() {
		Util.debug("Shake #" + this.cycleCount);

		if (!this.showPokeAlert) return;
		
		var self = this;

		$("#gadget-app").animate({left:self.startPos.x+(self.delta/2), top:self.startPos.y+self.delta}, self.speed, self.effect, function() {
			$("#gadget-app").animate({left:self.startPos.x-(self.delta/2), top:self.startPos.y+(self.delta/2)}, self.speed, self.effect, function() {
				$("#gadget-app").animate({left:self.startPos.x+self.delta, top:self.startPos.y+(self.delta/2)}, self.speed, self.effect, function() {
					$("#gadget-app").animate({left:self.startPos.x, top:self.startPos.y+self.delta}, self.speed, self.effect, function() {
						$("#gadget-app").animate({left:self.startPos.x, top:self.startPos.y}, self.speed, self.effect, function() {
							if(self.cycleCount++ < self.numCycles) {
								self.doShake();
							} else {
								self.cycleCount = 1;
							}
						});
					});
				});
			});
		});
		
	}
}