 /*
 +---------------------------------------------------------------------------+
 | Facebook Microsoft Vista Sidebar Gadget                                   |
 +---------------------------------------------------------------------------+
 | Copyright (c) 2007 Facebook, Inc.                                         |
 | All rights reserved.                                                      |
 |                                                                           |
 | Redistribution and use in source and binary forms, with or without        |
 | modification, are permitted provided that the following conditions        |
 | are met:                                                                  |
 |                                                                           |
 | 1. Redistributions of source code must retain the above copyright         |
 |    notice, this list of conditions and the following disclaimer.          |
 | 2. Redistributions in binary form must reproduce the above copyright      |
 |    notice, this list of conditions and the following disclaimer in the    |
 |    documentation and/or other materials provided with the distribution.   |
 |                                                                           |
 | THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR      |
 | IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES |
 | OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.   |
 | IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,          |
 | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  |
 | NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, |
 | DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     |
 | THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       |
 | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF  |
 | THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.         |
 +---------------------------------------------------------------------------+
 | Developer: Daniel Ruspini - facebook id dan@ruspini.com                   |
 +---------------------------------------------------------------------------+
*/

/*
 * Upload is the function literal controlling facebook upload image UI.
 */
Upload = new function() {

	this.stopped = true;	// whether drag-and-drop uploads are allowed
	this.maxDimensions = { width:0, height:0 };
	

	/*
	 *  Effectively enables drag-and-drop uploads
	 */
	this.start = function() {
		this.stopped = false;
	}

	/*
	 *  Effectively disables drag-and-drop uploads
	 */
	this.stop = function() {
		this.stopped = true;
	}
	
	/*
	 *  Hides the all upload app panes from view.
	 */
	this.hide = function() {
		this.hideProgress();
		this.hideResults();
		document.getElementById("upload").style.display = "none";
	}
	
	/*
	 *  Shows the upload app in its initial state after a drag-and-drop. 
	 *  Disable uploads and show the progress pane.
	 */
	this.show = function() {
		this.stop();
		document.getElementById("upload").style.display = "";
		this.showProgress();
	}
	
	/*
	 *  Hides the upload progress pane
	 */
	this.hideProgress = function() {
		document.getElementById("upload-progress").style.display = "none";
	}
	
	/*
	 *  Shows the upload progress pane and hides the results pane (in case its showing).
	 */
	this.showProgress = function() {
		Util.debug("[upload] showProgress");
		document.getElementById("upload-progress").style.display = "";
		this.hideResults();
	}
	
	/*
	 * Hides the upload results pane
	 */
	this.hideResults = function() {
		document.getElementById("upload-results").style.display = "none";
	}
	
	/*
	 *  Shows the upload results pane and hides the progress pane in case it's showing.
	 */
	this.showResults = function() {
		document.getElementById("upload-results").style.display = "";
		this.hideProgress();
	}
	
	/*
	 *  Control changes in view when gadget is docked.
	 */
	this.dock = function() {
		document.getElementById("upload-cancel").style.display = "none";
		this.maxDimensions.width = SlideShow.maxDimensions.width;
		this.maxDimensions.height = SlideShow.maxDimensions.height - 13;	//subtract height of upload control bar
		this.resizeCurrentImage();
	}
	
	/*
	 *  Control changes in view when gadget is undocked.
	 */
	this.undock = function() {
		document.getElementById("upload-cancel").style.display = "";
		this.maxDimensions.width = SlideShow.maxDimensions.width;
		this.maxDimensions.height = SlideShow.maxDimensions.height - 16;	//subtract height of upload control bar
		this.resizeCurrentImage();
	}
	
	/*
	 *  Send the uploading viewable image back for resizing.
	 *  Only called when an upload is in progress which we
	 *  might know by checking to see if uploads are disabled.
	 */
	this.resizeCurrentImage = function() {
		if (this.stopped) {	// essentially an upload is in progress
			Util.setStylesBasedOnImageDimensions(document.getElementById("upload-img"), this.maxDimensions, false);
		}
	}
	
	/*
	 *  Called by an onload event on the temp img node in the HTML.
	 *  When the image is fully loaded we'll scale it to size and
	 *  position it correctly.  Then we'll transfer the source and
	 *  styles to the real visible node.
	 */
	this.onLoadImage = function(imgNode) {
	    Util.setStylesBasedOnImageDimensions(imgNode, this.maxDimensions, false);
		imgNode.style.display = "";
	}
	
	/*
	 *  Is called when an uploaded image is being processed.
	 *  The image source is set to the temp img node which in turn
	 *  calls an onload event which handles the actually scaling
	 *  and transfer of src and styles to the visible image node.
	 */
	this.displayImage = function(path) {
		var uploadImgNode = document.getElementById("upload-img");
		uploadImgNode.style.display = "none";
		uploadImgNode.src = "file://" + path;
	}
	
	/*
	 *  Called at the beginning of an image upload so that 
	 *  UI displays information about image being uploaded.
	 */
	this.showPhotoUploadProgress = function(photoNumber, photoCount, file) {
	    Util.debug("Progress:" + photoNumber + " of " + photoCount + " file=" + file.stringValue());
	    this.displayImage(file.path);
	    document.getElementById("upload-progress-numbers").innerHTML = (photoNumber+1) + " of " + photoCount;
	}
	
	/*
	 *  Called after all images have been processed.
	 *  We display the results of the upload and enable drag-and-drop.
	 */
	this.onUnload = function(files) {
	    Util.debug("onUnload:");
		this.displayResults(files);
		document.getElementById("upload-img").src = "images/blank.gif";
		this.start();
	}
	
	/*
	 *  Update the UI to show upload success totals, 
	 *  send off the call to handle display of errors,
	 *  lastly show the results pane.
	 */
	this.displayResults = function(files) {
        Util.debug("displayResults: itemsUploadedCount="+files.itemsUploadedCount()+" length="+files.length());
		document.getElementById("uploaded-success").innerHTML = files.itemsUploadedCount();
		document.getElementById("uploaded-total").innerHTML = files.length();
		this.displayErrors(files);
		document.getElementById("approve-btn").style.display = files.itemsUploadedCount() == 0 ? "none" : "";	
		this.showResults();
	}

	/*
	 *  Passes upload errors to flyout and displays link on results pane.
	 */
	this.displayErrors = function(files) {
		Flyout.clearErrors();	// reset
		
		if (files.uploadErrors().length > 0) {
			Flyout.errors = files.uploadErrors();
			document.getElementById("upload-summary-msg").innerHTML = Util.getMessage("FileUploadErrorsFlyoutLinkText");
			document.getElementById("upload-errors").style.display = "";
		} else {
			document.getElementById("upload-summary-msg").innerHTML = "";
			document.getElementById("upload-errors").style.display = "none";
		}
	}
	
	/*
	 *  Called when the user clicks the "Approve Photos" link
	 */
	this.onClickManageUploadedPhotos = function() {
	    window.open(FacebookAPI.photoUploadAlbumLink);
	}
	
	/*
	 *  Called when the user clicks the cancel link.  
	 *  Sets a property inside PhotoUploadItemSummary which in turn is 
	 *  checked before continuing thru a loop of unprocessed files.
	 */
	this.onClickCancel = function() {
		PhotoUploadItemSummary.canceled = true;
	}
};