 /*
 +---------------------------------------------------------------------------+
 | Facebook Microsoft Vista Sidebar Gadget                                   |
 +---------------------------------------------------------------------------+
 | Copyright (c) 2007 Facebook, Inc.                                         |
 | All rights reserved.                                                      |
 |                                                                           |
 | Redistribution and use in source and binary forms, with or without        |
 | modification, are permitted provided that the following conditions        |
 | are met:                                                                  |
 |                                                                           |
 | 1. Redistributions of source code must retain the above copyright         |
 |    notice, this list of conditions and the following disclaimer.          |
 | 2. Redistributions in binary form must reproduce the above copyright      |
 |    notice, this list of conditions and the following disclaimer in the    |
 |    documentation and/or other materials provided with the distribution.   |
 |                                                                           |
 | THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR      |
 | IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES |
 | OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.   |
 | IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,          |
 | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  |
 | NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, |
 | DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     |
 | THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       |
 | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF  |
 | THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.         |
 +---------------------------------------------------------------------------+
 | Developer: Amine Nebri - facebook id anebri@yahoo.com                     |
 +---------------------------------------------------------------------------+
*/

/*
 * The Util class implements a set of helper functions in different
 * areas such as message logging, file uploading, Gadget settings, etc
 */
Util = new function()
{
    /*
     * Displays a log message
     */
    this.log = function(message)
    {
        System.Debug.outputString("FB v" + this.getGadgetVersion() + " [" +GadgetSetting.DebugLevel+ "]: " + message);
    }

    /*
     * Displays a the function name and log message and throws an error
     */
    this.error = function(functionName, message)
    {
        this.logErrorMessage(functionName, message);
        
        throw message;
    }
    
    /*
     * Returns true if debug is enabled. false otherwise.
     */
    this.isDebug = function()
    {
        return GadgetSetting.DebugLevel == "debug";
    }
    
    /*
     * Displays the error message in the debug console if
     * debug mode is enabled.
     */
    this.debug = function(message)
    {
        if (this.isDebug() == false)
            return;
            
        this.log(message);
    }
    
    /*
     * Displays an error message in the debug console using
     * the functionName and message arguments.
     */
    this.logErrorMessage = function(functionName, message)
    {   
        this.log(this.getErrorMessage(functionName, message));
    }
    
    /*
     * Returns the error message to display in the debug console
     * using the function name where the error occurent and a
     * message.
     */
    this.getErrorMessage = function(functionName, message)
    {
        return "EXCEPTION: Error in " + functionName + " Message:" + message;
    }

    /*
     * Returns the number of seconds since 1/1/1970.
     */
    this.getSecondsSince1970 = function()
    {
        return new Date().getTime();
    }
    
    /*
     * Builds a comma seprated list using the provided
     * array.
     */
    this.buildCommaSeparatedList = function(list)
    {
        this.debug("buildCommaSeparatedList: p="+list);

        var result = "";
    	
        for (var i=0; i<list.length; i++)
        {
            result += list[i];
            
            if (i != list.length-1)
                params+=",";
        }
        
        this.debug("buildCommaSeparatedList: params="+result);
        
        return result;
    }
    
    /*
     * Builds an url query string using the provided
     * array.
     */
    this.buildQueryString = function(p)
    {
        var params = "";
    	
        for (var i=0; i<p.length; i++)
        {
            params += p[i];
            
            if (i != p.length-1)
                params += "&";
        }
        
        return params;
    }
    
    /*
     * Pauses the current processing for the number
     * of milliseconds passed as an argument.
     */
    this.sleep = function(milliseconds)
    {
        var now = new Date();
        var endTime = now.getTime() + milliseconds;

        while(true)
        {
            now = new Date();
            if (now.getTime() > endTime)
                return;
        }
    }
    
    /*
     * Retrieves a gadget setting by key.
     */
    this.getSetting = function(key) 
    {
        return System.Gadget.Settings.readString(key);
    }

    /*
     * Stores the key/value pair in the gadget settings.
     */
    this.setSetting = function(key, value) 
    {
        System.Gadget.Settings.writeString(key, value);
    }
    
    /*
     * Remove the key from the gadget settings.
     */
    this.removeSetting = function(key) 
    {
        System.Gadget.Settings.writeString(key, "");
    }
    
    /*
     * Returns a binary stream.
     * It is the responsibility of the call to close
     * the stream.
     */
    this.getBinaryStream = function(path)
    {
        if (path == undefined || path == "")
            this.error("getBinaryStream", "path cannot be empty");
        
        var stream = new ActiveXObject("ADODB.Stream");
        
        stream.Type = 1;
        stream.Open;
        stream.LoadFromFile(path);

        return stream;
    }
    
    /*
     * Uploads a binary file to facebook and returns an AjaxResponse object.
     * The file is uploaded using the facebook.photos.upload API call that
     * requires building a multipart http request.
     * An exception is thrown if an error is encountered.
     * This function requires a session.
     */
    this.uploadBinaryFile = function(url, filePath, mimeType)
    {
        this.debug("uploadBinaryFile: filePath=" + filePath + " mimeType="+mimeType);
       
        try
        {
            var stream = this.getBinaryStream(filePath);
            var response = uploadBinaryFile(url, stream, filePath, mimeType);
                      
            stream.close();
            stream = null;
                
            return response;
        }
        catch (ex)
        {
            this.error("uploadBinaryFile", "Error uploading binary file exception="+ex);
        }
    }
    
    /*
     * Returns a message based on the provided
     * error code. Error Messages are defined in GadgetSetting.Messages.
     */
    this.getMessage = function(errorCode)
    {    
        for (var i=0; i<GadgetSetting.Messages.length; i++) {
            if (GadgetSetting.Messages[i].code == errorCode) {
            	if (GadgetSetting.Messages[i].message.docked != undefined && System.Gadget.docked) {
            		return GadgetSetting.Messages[i].message.docked;
            	} else if (GadgetSetting.Messages[i].message.undocked != undefined && !System.Gadget.docked) {
            		return GadgetSetting.Messages[i].message.undocked;
            	} else {
					return GadgetSetting.Messages[i].message;
				}
			}
		}
    }
    
    /*
     * Parses a string a JSON object and returns the JavaScript
     * object.
     */
    this.getJSON = function(json)
    {
        if (json == undefined || json == "")
            return null;

        try
        {
            if (/^("(\\.|[^"\\\n\r])*?"|[,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t])+?$/.test(json))
                return eval('('+json+')');
        }
        catch (ex)
        {
            //Swallow
        }
        
        this.error("getJSON", "Unable to parse JSON string \"" + json + "\" ex="+ex);
    }
    
    /*
     * Returns the version of the gadget.
     */
    this.getGadgetVersion = function()
    {
        return System.Gadget.version;
    }
    
    /*
     * Prints system info of the gadget including version and
     * hardware configuration.
     */
    this.printGadgetInfo = function()
    {
        this.log("-----------------------------------------------------------------");
        this.log(System.Gadget.name + " Vista Gadget Version=" + this.getGadgetVersion() + " PlatformVersion=" + System.Gadget.platformVersion);
        this.log("Processor="+System.Machine.processorArchitecture + ", "+System.Machine.CPUs.count + " CPU(s), CPU Name="+System.Machine.CPUs.item(0).name);
        this.log("-----------------------------------------------------------------");
        this.log("");
    }
    
    /*
     * Returns true if big string starts with small string. false otherwise.
     */
    this.startsWith = function(big, small)
    {
        return big.indexOf(small)==0;
    }
    
    /*
     * Adds item to the array if the item does not
     * already exist.
     */
    this.pushIfMissing = function(array, item)
    {
        var found = false;
        
        for (var i=0; i<array.length; i++)
            if (array[i] == item)
            {
                found = true;
                break;
            }
                
        if (found == false)
            array.push(item);    
    }
    
    /*
     * Returns the index if item in the provided array.
     */
    this.getIndexOf = function(array, item)
    {
        for (var i=0; i<array.length; i++)
            if (array[i] == item)
                return i;
                
        return -1;
    }
    
    /*
     * Prints the content of the array in the debug console.
     */
    this.printArray = function(array)
    {
        this.debug("printArray: array Count="+array.length);
        
        for (var i=0; i<array.length; i++)
            this.debug("printArray: i="+i+" value="+ array[i]);
    }
    
    /*
     * Returns true if str only contains alphanumeric characters. false otherwise.
     */
    this.isAlphanumeric = function(str)
    {
        if (str == undefined || str == "")
            return false;
            
        if (GadgetSetting.AlphanumericRegex.test(str) == true)
            return true;
            
        return false;
    }
    
    /*
     * Returns an MD5 encoded string using str as an argument
     */
    this.getMD5EncodedString = function(str)
    {       
        return hex_md5(str);
    }
    
	this.setStylesBasedOnImageDimensions = function(imgNode, maxDimensions, doCrop) {
		var temp = new Image();
		temp.width = imgNode.width;
		temp.height = imgNode.height;
		temp.src = imgNode.src;

		// Reset styles we may change
		imgNode.style.marginLeft = "0";
		imgNode.style.marginTop = "0";
		imgNode.style.width = "";
		imgNode.style.height = "";
		imgNode.style.left = "0";
		imgNode.style.top = "0";
		
		if (temp.width > temp.height) {
			// landscape
			if (doCrop) {
				imgNode.style.height = maxDimensions.height + "px";
				var adjustedWidth = maxDimensions.height * temp.width / temp.height;
				imgNode.style.width = adjustedWidth + "px";
				if (adjustedWidth > maxDimensions.width) {
					var adjustedLeftPosition = -1 * Math.round((adjustedWidth - maxDimensions.width) / 2);
					imgNode.style.left = adjustedLeftPosition + "px";
				} else {
					// adjusted width is still not wide enough to fill frame (this can really only happen in an undocked state)
					imgNode.style.width = maxDimensions.width;
					imgNode.style.height = maxDimensions.width * maxDimensions.height / adjustedWidth;
				}				
			} else {
				imgNode.style.width = maxDimensions.width + "px";
				var adjustedHeight = maxDimensions.width * temp.height / temp.width;
				imgNode.style.height = adjustedHeight + "px";
				if (adjustedHeight < maxDimensions.height) {
					imgNode.style.marginTop = ((maxDimensions.height - adjustedHeight) / 2) + "px";
				}
			}
		} else if (temp.height > temp.width) {
			// portrait
			if (doCrop) {
				imgNode.style.width = maxDimensions.width + "px";
				var adjustedHeight = maxDimensions.width * temp.height / temp.width;
				imgNode.style.height = adjustedHeight + "px";
				if (adjustedHeight > maxDimensions.height) {
					var adjustedTopPosition = -1 * Math.round((adjustedHeight - maxDimensions.height) / 2) -2;
					imgNode.style.top = adjustedTopPosition + "px";
				}
			} else {
				var adjustedWidth = maxDimensions.height * temp.width / temp.height;
				imgNode.style.width = adjustedWidth + "px";
				imgNode.style.height = maxDimensions.height + "px";
				if (adjustedWidth < maxDimensions.width) {
					imgNode.style.marginLeft = ((maxDimensions.width - adjustedWidth) / 2) + "px";
				}
			}
		} else {
			// square
			imgNode.style.width = maxDimensions.width + "px";
			imgNode.style.height = maxDimensions.height + "px";
		}
		
	}
}
